Imports System.ComponentModel
Imports System.Diagnostics

Namespace Misc

   Public Class ComplexConverter

      Private Const Separator As Char = Microsoft.VisualBasic.ChrW(&HF019)

      ''' <summary> in ApplyDatastring werden alle Exceptions gecatcht - ausser diese </summary>
      Public Class ComplexConverterException : Inherits Exception
         Public Sub New(msg As String, Optional ex As Exception = Nothing)
            MyBase.New(msg, ex)
         End Sub
      End Class

      Public Class EventArg : Inherits EventArgs

         ' Das Event ComplexConverter.Convert(), welches dieses EventArg verwendet, wird in zwei
         ' Modi ausgefhrt:
         ' Store konvertiert einen gegebenen Wert und fgt der IList(Of String) einen Eintrag hinzu
         ' Restore re-konvertiert einen Eintrag und berschreibt damit den gegebenen Wert

         Private _Strings As IList(Of String)
         Private _Counter As Integer = 0
         Public ReadOnly IsStoring As Boolean

         Sub New( Strings As IList(Of String),  IsStoring As Boolean)
            _Strings = Strings
            Me.IsStoring = IsStoring
         End Sub

         ''' <summary>Hier angegebene Werte werden gespeichert und restauriert</summary>
         <Description("Hier angegebene Werte werden gespeichert und restauriert")> _
         Public Sub ConvertValue(Of T)(ByRef arg As T)
            Dim o As Object = arg
            ConvertExplicitTyped(Of T)(o)
            arg = DirectCast(o, T)
         End Sub
         ''' <summary>Hier angegebene Werte werden gespeichert und restauriert</summary>
         <Description("Hier angegebene Werte werden gespeichert und restauriert")>
         Public Sub ConvertValue(Of T1, T2)(ByRef arg1 As T1, ByRef arg2 As T2)
            ConvertValue(arg1) : ConvertValue(arg2)
         End Sub
         ''' <summary>Hier angegebene Werte werden gespeichert und restauriert</summary>
         <Description("Hier angegebene Werte werden gespeichert und restauriert")>
         Public Sub ConvertValue(Of T1, T2, T3)(ByRef arg1 As T1, ByRef arg2 As T2, ByRef arg3 As T3)
            ConvertValue(arg1, arg2) : ConvertValue(arg3)
         End Sub
         ''' <summary>Hier angegebene Werte werden gespeichert und restauriert</summary>
         <Description("Hier angegebene Werte werden gespeichert und restauriert")>
         Public Sub ConvertValue(Of T1, T2, T3, T4)(ByRef arg1 As T1, ByRef arg2 As T2, ByRef arg3 As T3, ByRef arg4 As T4)
            ConvertValue(arg1, arg2) : ConvertValue(arg3, arg4)
         End Sub

         ''' <summary>Hier angegebene Werte werden gespeichert und restauriert. Der Datentyp ist anzugeben</summary>
         <Description("Hier angegebene Werte werden gespeichert und restauriert")> _
         Public Sub ConvertExplicitTyped(Of T)(ByRef Item As Object)
            If IsStoring Then ConvertExplicitTyped(Item, GetType(T)) Else Item = GetValue(Of T)()
         End Sub
         ''' <summary>Hier angegebene Werte werden gespeichert und restauriert. Der Datentyp ist anzugeben</summary>
         <Description("Hier angegebene Werte werden gespeichert und restauriert")> _
         Public Sub ConvertExplicitTyped(ByRef Item As Object, propType As Type)
            If IsStoring Then
               Dim s = TryCast(Item, String)
               If s Is Nothing Then
                  s = TypeDescriptor.GetConverter(propType).ConvertToString(Item)
               ElseIf s.Contains(Separator) Then
                  Dim msg As String = String.Concat( _
                      "Der zu speichernde String '", s, _
                      "' darf nicht den ComplexConverter.Separator-Char (", _
                      System.Convert.ToString(Microsoft.VisualBasic.AscW(Separator), 16), _
                      ") enthalten.")
                  Throw New ArgumentException(msg, "Item")
               End If
               _Strings.Add(s)
            Else
               Item = GetValue(propType)
            End If
         End Sub

         ''' <summary>gibt den nchsten gespeicherten Wert zurck</summary>
         <Description("gibt einen gespeicherten Wert zurck")> _
         Public Function GetValue(Of T)() As T
            Return DirectCast(GetValue(GetType(T)), T)
         End Function
         ''' <summary>gibt einen gespeicherten Wert zurck</summary>
         <Description("gibt einen gespeicherten Wert zurck")> _
         Public Function GetValue(propType As Type) As Object
            If IsStoring Then
               Throw New ComplexConverterException( _
                   ".GetValue() darf nicht aufgerufen werden, wenn .IsStoring = True.")
            Else
               Dim C As TypeConverter = TypeDescriptor.GetConverter(propType)
               GetValue = C.ConvertFromString(_Strings(_Counter))
               _Counter += 1
            End If
         End Function

         ' Sonderfall ConvertList(): Bei Listen wird die Anzahl Count der Elemente gespeichert.
         ' Restore leert die Liste und fgt dann Count uninitialisierte Elemente hinzu.

         ''' <summary>speichert Anzahl, restauriert mit uninitialisierten Elementen</summary>
         <Description("speichert Anzahl, restauriert mit uninitialisierten Listenelementen")> _
         Public Sub ConvertList(Of TItem As New)(List As System.Collections.IList)
            If IsStoring Then
               _Strings.Add(List.Count.ToString)
            Else
               List.Clear()
               For I As Integer = 0 To Integer.Parse(_Strings(_Counter)) - 1
                  List.Add(New TItem)
               Next
               _Counter += 1
            End If
         End Sub

         Public Sub ConvertConverter( Converter As ComplexConverter)
            ' ComplexConverters listigste Zeile: ermglicht, den Konvertierungs-Vorgang an einen
            ' anderen Converter abzugeben, dabei aber "meine" _Strings zu verwenden. So kann ein
            ' UserControl aus einer anderen Assembly seine eigene Konvertierung definieren, die
            ' dann in den Settings der Entry-Assembly mit-persistiert wird
            Converter.OnConvert(Me)
         End Sub

      End Class ' ComplexConverter.EventArg

      Public Event Convert As EventHandler(Of EventArg)

      Private Sub OnConvert( e As EventArg)
         RaiseEvent Convert(Me, e)
      End Sub

      Public Function CreateDataString() As String
         Dim Strings As New List(Of String)
         RaiseEvent Convert(Me, New EventArg(Strings, True))
         Return String.Join(Separator, Strings.ToArray)
      End Function

      Public Function ApplyDataString( S As String) As Boolean
         If S = "" Then Debug.WriteLine("no data to restore") : Return False
         Dim Backup As New List(Of String)
         OnConvert(New EventArg(Backup, IsStoring:=True))
         Try
            'Restaurierung anhand von S versuchen
            OnConvert(New EventArg(S.Split(Separator), False))
            Return True
         Catch Ex As ComplexConverterException
            Throw
         Catch Ex As Exception
            'Im Fehlerfall Re-Restaurierung anhand des Backups
            Debug.WriteLine("Restoring from data failed")
            OnConvert(New EventArg(Backup, False))
         End Try
         Return False
      End Function


   End Class

End Namespace
