﻿#Region "FileHeader"
#If False Then
Die Winz-Klasse GraphAttach(Of T) ordnet jedem Objekt ein Parents- und ein Childs- Hashset zu. ConnectChild() im anschliessenen Modul nutzt das, um beliebige gleichartige Objekte als bidirektionalen Graphen (gerichtete, aber ungewichtete Kanten) miteinander zu verknüpfen
Der Graph ist mathematisch eigentümlich, denn er hat nur implizit Kanten (Edges). Ein richtiger Graph bestünde im Wesentlichen nur aus einer einzigen Auflistung von Edges, deren jede 2 Objekte (Nodes) verknüpfte. Um in einem richtigen Graphen von einem Node alle ChildNodes zu erhalten muß man alle Edges durchlaufen und diejenigen selecten, bei denen der Node als Parent eingetragen ist - und davon je den Childnode nehmen. Das wäre für Traversen imperformant, und man würde zur Optimierung ein Dictionary bilden, was zu jedem Node 2 Edges-Auflistung bereitstellt: Child-Edges und Parent-Edges. Haben die Edges nun keine weitere Information (ungewichtet), so können sie auch weggelassen werden, und stattdessen kann man die gegenüberliegenden Nodes auch direkt in die Auflistungen eintragen.
Genau das liegt hier vor: Das Dictionary ist die Shared ConditionalWeakTable der Basisklasse, und die Auflistungen der Parents/Childs sind hier als Hashset ausgeführt, um Doppel-Einträge beim connecten auszuschliessen.

Ausserdem gibts hier noch Algos für BreathFirst-Traversion, für FloodFill und TopologicSort, je mit auch einer Variante, die den attachten Graphen nutzt.
Bemerkenswert, dass keine dieser Traversen wirklich beide Kanten-Richtungen benötigt - selbst eine topologische Sortierung liesse sich auch aus der Parent->Child-Richtung bilden - man müsste nur die Ausgabe-Reihenfolge invertieren
#End If '-- Options, Imports
#End Region 'FileHeader

Imports System.Runtime.CompilerServices
Namespace GraphAttachment

   Public Class GraphAttach(Of T As Class) : Inherits WeakAttachBase(Of GraphAttach(Of T))
      Public ReadOnly Parents As New HashSet(Of T)
      Public ReadOnly Childs As New HashSet(Of T)
   End Class ' GraphAttach

   Public Module GraphAttachX
      <Extension> _
      Public Function Parents(Of T As Class)(itm As T) As HashSet(Of T)
         Return GraphAttach(Of T).Get(itm).Parents
      End Function
      <Extension> _
      Public Function Childs(Of T As Class)(itm As T) As HashSet(Of T)
         Return GraphAttach(Of T).Get(itm).Childs
      End Function
      ''' <summary>attacht eine bidirektionale Verknüpfung zw. itm und chld, dabei chld als untergeordnet </summary>
      <Extension> _
      Public Sub ConnectChild(Of T As Class)(itm As T, chld As T)
         itm.Childs.Add(chld)
         chld.Parents.Add(itm)
      End Sub

      <Extension()> _
      Public Function BreathFirst(Of T As Class)(Roots As IEnumerable(Of T)) As IEnumerable(Of T)
         Return BreathFirst(AddressOf Childs(Of T), Roots)
      End Function
      <Extension()> _
      Public Iterator Function BreathFirst(Of T)(getSuccessors As Func(Of T, IEnumerable(Of T)), Roots As IEnumerable(Of T)) As IEnumerable(Of T)
         Dim q As New Queue(Of T)(Roots)
         While q.Count > 0
            Dim itm = q.Dequeue
            Yield itm
            getSuccessors(itm).ForEach(AddressOf q.Enqueue)
         End While
      End Function

      <Extension()> _
      Public Function FloodFill(Of T As Class)(Roots As IEnumerable(Of T)) As HashSet(Of T)
         Return FloodFill(AddressOf Childs(Of T), Roots)
      End Function
      <Extension()> _
      Public Function FloodFill(Of T)(getSuccessors As Func(Of T, IEnumerable(Of T)), Roots As IEnumerable(Of T)) As HashSet(Of T)
         'FloodFill ist ein gegen Kreisläufe gesicherter BreathFirst. Optimiert würden die durchlaufenen Items markiert - stattdessen - allgemeingültig - wird hier ein Hashset verwendet.
         Dim ret As New HashSet(Of T)(Roots)
         Dim q As New Queue(Of T)(ret)
         While q.Count > 0
            getSuccessors(q.Dequeue).Where(AddressOf ret.Add).ForEach(AddressOf q.Enqueue)
         End While
         Return ret
      End Function

      <Extension()> _
      Public Function TopologicSort(Of T As Class)(items As IEnumerable(Of T)) As List(Of T)
         Return TopologicSort(AddressOf Parents(Of T), items)
      End Function
      <Extension()> _
      Public Function TopologicSort(Of T)(getParents As Func(Of T, IEnumerable(Of T)), items As IEnumerable(Of T)) As List(Of T)
         Dim hsh As New HashSet(Of T)(items)
         TopologicSort = New List(Of T)
         Dim recurse As Action(Of T) = _
            Sub(itm)
            If Not hsh.Remove(itm) Then Return
            getParents(itm).ForEach(recurse)
            TopologicSort.Add(itm)
         End Sub
         While hsh.Count > 0 : recurse(hsh.First()) : End While
      End Function

   End Module
End Namespace
