﻿Imports System.Text.RegularExpressions
Imports System.Diagnostics

#If False Then
Zu Codepages siehe auch https://de.wikipedia.org/wiki/Kategorie:Zeichenkodierung
Encoding.Default            'Ansi (CP1252 bei deutschem Windows) - zb Excel-csv
Encoding.GetEncoding(1252)  'Ansi
Encoding.UTF8               'UTF8
Encoding.GetEncoding(65001) 'UTF8
Encoding.Unicode            'UTF16LE
Encoding.GetEncoding(437)   'OEM (DOS) 437                  - zb Konsole-Ausgaben
Encoding.GetEncoding(850)   '437 - bischen weiterentwickelt - zb Excel-csv
Encoding.ASCII              'Nur 7-Bit Zeichen (bis Zeichen 127)
#End If

Imports System.Runtime.CompilerServices
Imports System.Text
Imports System.Xml.Serialization

Namespace System.IO

	Public Module SystemIOX

		Public ReadOnly Utf8NoBom As New Text.UTF8Encoding(False)

		Private _PathReplacer As MatchEvaluator = Function(mt As Match) IO.Path.GetFullPath(mt.Groups(1).Value)
		Private _QuotedReplacer As MatchEvaluator = Function(mt As Match) """".And(Path.GetFullPath(mt.Groups(1).Value), """")
		Private Function CreateQuotedDirPattern() As Regex
			Dim escapeds = From c In Path.GetInvalidPathChars Select "\x" & Convert.ToInt32(c).ToString("x2")
			Return New Regex("""?((\.{2}\\)+[^;".And(String.Concat(escapeds), "]+)""?"))
		End Function
		<Extension()>
		Public Function QuotedFullnamed(s As String) As String
			Return CreateQuotedDirPattern.Replace(s, _QuotedReplacer)
		End Function
		<Extension()>
		Public Function Fullnamed(s As String) As String
			Return CreateQuotedDirPattern.Replace(s, _PathReplacer)
		End Function
		''' <summary>Falls der Pfad escaped werden musste, werden noch ein Mitteilungs-Text und eine Mitteilungs-Überschrift mitgegeben </summary>
		<Extension()>
		Public Function EscapePath(s As String) As String()
			Dim splits = s.Split(Path.GetInvalidPathChars)
			Dim s2 = splits(0)
			If splits.Length = 1 Then Return {s2}
			s2 = "_".Between(splits)
			Return {s2, "vorher:".And2(Lf, s, "\nnachher:", Lf, s2), "Illegale Zeichen wurden aus Dateipfad entfernt"}
		End Function

		<Extension(), DebuggerStepThrough()>
		Public Function Combine(di As DirectoryInfo, ParamArray segments As String()) As String
			Dim strings(segments.Length) As String
			strings(0) = di.FullName
			Array.Copy(segments, 0, strings, 1, segments.Length)
			Return Path.GetFullPath(Path.Combine(strings))
		End Function


#If True Then
		Private _rgxDigits As New Regex("\d*?(?=\.[^\.]*$|$)")

		<Extension()>
		Public Function NumberedFullname(fi As FileInfo) As String
			Dim mt = _rgxDigits.Match(fi.Name)
			Dim name = fi.Name.Substring(0, mt.Index + mt.Length)
			Select Case mt.Length
				Case 2 : name = name.SubstringX(-2) 'digits ersetzen
				Case 0 'digits anhängen
				Case Else : name &= "_" 'digits anhängen mit '_' zwischen
			End Select
			Dim di = fi.Directory
			Dim fis = di.GetFiles(name.And("??", fi.Extension))
			Dim i = 0
			Dim imax = If(fis.Length = 0, -1, Aggregate f In fis Where Integer.TryParse(f.Name.Substring(name.Length, 2), i) Into Max(i))
			Return di.FullName & "\{}{:00}{}".FormatX(name, imax + 1, fi.Extension)
		End Function
#ElseIf True Then
		<Extension()> _
		Public Function NumberedFileName(fi As FileInfo) As String
			Dim name = fi.Name.SubstringX(-fi.Extension.Length)
			Dim shortenedNam = name.SubstringX(-2)
			If name Like shortenedNam & "##" Then name = shortenedNam

			Dim di = fi.Directory
			Dim fis = di.GetFiles(name.And("??", fi.Extension))
			Dim i = 0
			Dim imax = If(fis.Length = 0, -1, Aggregate f In fis Where Integer.TryParse(f.Name.Substring(name.Length, 2), i) Into Max(i))
			Return di.FullName & "\{}{:00}{}".FormatX(name, imax + 1, fi.Extension)
		End Function
#ElseIf True Then
#End If


		<Extension(), DebuggerStepThrough()>
		Public Function GetFullnameWithoutExtension(fi As FileInfo) As String
			Return GetFullnameWithoutExtension(fi.FullName)
		End Function
		<Extension(), DebuggerStepThrough()>
		Public Function GetFilenameWithoutExtension(fi As FileInfo) As String
			Return Path.GetFileNameWithoutExtension(fi.Name)
		End Function
		<Extension(), DebuggerStepThrough()>
		Public Function ReadAllText(fi As FileInfo) As String
			Return File.ReadAllText(fi.FullName)
		End Function
		<Extension(), DebuggerStepThrough()>
		Public Sub WriteAllText(fi As FileInfo, txt As String)
			File.WriteAllText(fi.FullName, txt)
		End Sub
		<Extension()>
		Public Function ReadAllLines(fi As FileInfo) As List(Of String)
			Dim Lst As New List(Of String)
			Using sr = fi.OpenText
				Do Until sr.EndOfStream
					Lst.Add(sr.ReadLine)
				Loop
			End Using
			Return Lst
		End Function

		Public Function GetFullnameWithoutExtension(s As String) As String
			Dim l = Path.GetExtension(s).Length
			If Not s.Contains(":") Then s = Path.GetFullPath(s)
			Return s.Substring(0, s.Length - l)
		End Function


		''' <summary> kopiert von einem Stream in einen anderen </summary>
		''' <remarks> 
		''' Es gibt Streams ohne festgelegtes Ende (zB. NetworkStream).
		''' In solchem Fall **muß** 'count' angegeben werden.
		''' </remarks>
		<Extension()>
		Public Sub WriteTo(
					  readStream As Stream,
					  writeStream As Stream,
					 Optional count As Long = -1,
					 Optional bufSize As Integer = 4096)
			Dim buf(bufSize - 1) As Byte
			If count < 0 AndAlso readStream.CanSeek Then
				count = readStream.Length - readStream.Position
			End If
			If count < 0 Then
				' Durch 0-Byte-Lesevorgang terminierte Kopier-Schleife
				' Ein NetworkStream würde ein Timeout-Problem verursachen
				Do
					Dim portion = readStream.Read(buf, 0, bufSize)
					If portion = 0 Then Return
					writeStream.Write(buf, 0, portion)
				Loop
			Else
				' zähler-gesteuerte Kopier-Schleife
				Do
					If count < bufSize Then bufSize = CInt(count)
					Dim portion = readStream.Read(buf, 0, bufSize)
					If portion = 0 Then Throw New ArgumentException(
						"Die angegebene Anzahl Bytes konnte nicht aus dem Lese-Stream gelesen werden",
						"readStream + count")
					count -= portion
					writeStream.Write(buf, 0, portion)
				Loop Until count = 0
			End If
		End Sub
		''' <summary>Achtung! FileInfo.OpenWrite ist ungeeignet, denn es erneuert nicht unbedingt die ganze Datei</summary>
		<Extension()>
		Public Sub SerializeXml(fi As FileInfo, obj As Object)
			Using strm = fi.Open(FileMode.Create)
				strm.SerializeXml(obj)
			End Using
		End Sub
		<Extension()>
		Public Sub SerializeXml(strm As Stream, obj As Object)
			Call (New XmlSerializer(obj.GetType)).Serialize(strm, obj)
		End Sub
		<Extension()>
		Public Function DeserializeXml(Of T)(fi As FileInfo) As T
			Using strm = fi.OpenRead
				Return strm.DeserializeXml(Of T)()
			End Using
		End Function
		<Extension()>
		Public Function DeserializeXml(Of T)(strm As Stream) As T
			Return DirectCast((New XmlSerializer(GetType(T))).Deserialize(strm), T)
		End Function

		''' <summary>Achtung! FileInfo.OpenWrite ist ungeeignet, denn es erneuert nicht unbedingt die ganze Datei</summary>
		<Extension()>
		Public Sub Serialize(fi As FileInfo, obj As Object)
			Using strm = fi.Open(FileMode.Create)
				strm.Serialize(obj)
			End Using
		End Sub
		<Extension()>
		Public Sub Serialize(strm As Stream, obj As Object)
			Call (New Runtime.Serialization.Formatters.Binary.BinaryFormatter).Serialize(strm, obj)
		End Sub
		<Extension()>
		Public Function Deserialize(Of T)(fi As FileInfo) As T
			Using strm = fi.OpenRead
				Return strm.Deserialize(Of T)()
			End Using
		End Function
		<Extension()>
		Public Function Deserialize(Of T)(strm As Stream) As T
			Return DirectCast((New Runtime.Serialization.Formatters.Binary.BinaryFormatter).Deserialize(strm), T)
		End Function

	End Module

End Namespace