﻿Imports System.Runtime.CompilerServices
Imports System.Reflection
Imports System.Diagnostics
'Imports Helpers.BitOperation
Namespace System.ComponentModel
	'NotifyPropertyChanged
	<Serializable>
	Public Class NotifyPropertyChanged : Implements INotifyPropertyChanged, ICloneable

		'Wpf-Version: DesignerProperties.GetIsInDesignMode(DependancyObject)
		Private Shared _IsInDesignMode As Boolean? = Nothing
		Private Shared _HostProcesses As New List(Of String)({"XDesProc", "devenv", "WDExpress"})
		Public Shared ReadOnly Property IsInDesignMode() As Boolean
			Get
				If Not _IsInDesignMode.HasValue Then
					_IsInDesignMode = _HostProcesses.Contains(Process.GetCurrentProcess().ProcessName)
				End If
				Return _IsInDesignMode.Value
			End Get
		End Property
		Public Shared Sub MockDesignmodeAs(value As Boolean)
			_IsInDesignMode = value
		End Sub

		<NonSerialized>
		Private _PropertyChanged As PropertyChangedEventHandler
		Public Custom Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
			AddHandler(value As PropertyChangedEventHandler)
				_PropertyChanged = DirectCast([Delegate].Combine(_PropertyChanged, value), PropertyChangedEventHandler)
			End AddHandler
			RemoveHandler(value As PropertyChangedEventHandler)
				_PropertyChanged = DirectCast([Delegate].Remove(_PropertyChanged, value), PropertyChangedEventHandler)
			End RemoveHandler
			RaiseEvent(sender As Object, e As PropertyChangedEventArgs)
				If _PropertyChanged IsNot Nothing Then _PropertyChanged(sender, e)
			End RaiseEvent
		End Event

		Protected Function ChangePropIfDifferent(Of T)(value As T, ByRef backingField As T, <CallerMemberName> Optional name As String = "") As Boolean
			Return ChangePropIfDifferent(value, backingField, {New PropertyChangedEventArgs(name)})
		End Function
		Protected Function ChangePropIfDifferent(Of T)(value As T, ByRef backingField As T, ParamArray propNames() As String) As Boolean
			Return ChangePropIfDifferent(value, backingField, propNames.ToArray(Function(s) New PropertyChangedEventArgs(s)))
		End Function
		'Protected Function ChangePropIfDifferent(Of T)(value As T, ByRef backingField As T, callbackIfChanged As Action) As Boolean
		'   If Object.Equals(backingField, value) Then Return False
		'   backingField = value
		'   callbackIfChanged()
		'   Return True
		'End Function
		Protected Function ChangePropIfDifferent(Of T)(value As T, ByRef backingField As T, ParamArray props() As PropertyChangedEventArgs) As Boolean
			If Object.Equals(backingField, value) Then Return False
			backingField = value
			RaisePropChanged(props)
			Return True
		End Function
		Public Sub RaisePropChanged(Optional name As String = "")
			RaisePropChanged({New PropertyChangedEventArgs(name)})
		End Sub
		Public Sub RaisePropChanged(name As String, ParamArray names() As String)
			Dim names2(names.Length) As String
			names2(0) = name : Array.Copy(names, 0, names2, 1, names.Length)
			RaisePropChanged(names2.ToArray(Function(nm) New PropertyChangedEventArgs(nm)))
		End Sub

		Public Sub RaisePropChanged(ParamArray eventArgses() As PropertyChangedEventArgs)
			RaisePropChanged(DirectCast(eventArgses, IEnumerable(Of PropertyChangedEventArgs)))
		End Sub
		Public Sub RaisePropChanged(eventArgses As IEnumerable(Of PropertyChangedEventArgs))
			VerifyPropertyNames(eventArgses)
			For Each e In eventArgses
				OnPropertyChanged(e)
			Next
		End Sub
		Public Sub RaiseAllPropChanged()
			OnPropertyChanged(New PropertyChangedEventArgs(Nothing))
		End Sub
		Protected Overridable Sub OnPropertyChanged(e As PropertyChangedEventArgs)
			RaiseEvent PropertyChanged(Me, e)
		End Sub

		Private Shared _AllProperties As New Dictionary(Of Type, HashSet(Of String))
		<Conditional("DEBUG"), DebuggerStepThrough()>
		Public Sub VerifyPropertyNames(eventArgses As IEnumerable(Of PropertyChangedEventArgs))
			Dim propNames As HashSet(Of String) = Nothing
			Dim tp = Me.GetType
			If Not _AllProperties.TryGetValue(tp, propNames) Then
				propNames = New HashSet(Of String)(From pd In Me.GetType.GetProperties Select pd.Name)
				propNames.Add("")
				_AllProperties.Add(tp, propNames)
			End If
			For Each pe In eventArgses
				If Not propNames.Contains(pe.PropertyName) Then Throw New Exception("trying to raise an unknown PropertyChanged")
			Next
		End Sub

		''' <summary>Überschreibungen werden auch von ICloneable.Clone() aufgerufen</summary>
		Public Overridable Sub CopyTo(other As Object)
		End Sub
		Public Function Clone() As Object Implements ICloneable.Clone
			Dim cl = DirectCast(MemberwiseClone(), NotifyPropertyChanged)
			cl._PropertyChanged = Nothing
			CopyTo(cl)
			Return cl
		End Function
	End Class

End Namespace
