﻿Imports System.Runtime.InteropServices

Namespace System

   ''' <summary>
   ''' encapsulates converting of multiple valuetypes and valuetype-arrays (even multidimensional) to a byte-array (Blob)
   ''' </summary>
   Public NotInheritable Class Blobber
      Private _Offset As Integer = 0
      Private _Bytes As Byte()

      Public Sub New(Bytes As Byte())
         _Bytes = Bytes
      End Sub

      ''' <summary>writes sequential all args into one byte-array and returns it. args must be valueTypes or (even multidimensional) arrays of valueTypes</summary>
      Public Shared Function CreateBlob(ParamArray args As Object()) As Byte()
         'evaluate required space of each arg (sum overall-space into offset)
         Dim offset = 0
         Dim lengths(args.Length - 1) As Integer
         For i = 0 To args.Length - 1
            Dim tp = args(i).GetType
            Dim length = 1
            If tp.IsArray Then
               tp = tp.GetElementType
               length = DirectCast(args(i), Array).Length
            Else
               Dim box = Array.CreateInstance(tp, 1)
               box.SetValue(args(i), 0)
               args(i) = box
            End If
            length *= Marshal.SizeOf(tp)
            lengths(i) = length
            offset += length
         Next
         Dim retVal(offset - 1) As Byte
         'write args into retVal
         offset = 0
         For i = 0 To args.Length - 1
            With GCHandle.Alloc(args(i), GCHandleType.Pinned)
               Marshal.Copy(.AddrOfPinnedObject, retVal, offset, lengths(i))
               .Free()
            End With
            offset += lengths(i)
         Next
         Return retVal
      End Function

      ''' <summary>Reads a T from Blob. T must be valueType</summary>
      Public Function Read(Of T)() As T
         Dim box(0) As T
         ReadTo(box)
         Return box(0)
      End Function
      ''' <summary>Reads an object from Blob. tp must represent valueType</summary>
      Public Function Read(tp As Type) As Object
         Dim box = Array.CreateInstance(tp, 1)
         ReadTo(tp, box)
         Return box.GetValue(0)
      End Function
      ''' <summary>
      ''' Fills destination from Blob. T must be valueType. destination may be multidimensional
      ''' </summary>
      Public Sub ReadTo(Of T)(destination As T())
         ReadTo(GetType(T), destination)
      End Sub
      ''' <summary>
      ''' Fills destination from Blob. tp must represent a valueType. destination may be multidimensional
      ''' </summary>
      Public Sub ReadTo(tp As Type, destination As Array)
         Dim size = Marshal.SizeOf(tp) * destination.Length
         With GCHandle.Alloc(destination, GCHandleType.Pinned)
            Marshal.Copy(_Bytes, _Offset, .AddrOfPinnedObject, size)
            .Free()
         End With
         _Offset += size
      End Sub

   End Class

End Namespace