﻿Imports System.Runtime.CompilerServices
Imports System.Diagnostics
Imports System.IO
Imports Newtonsoft.Json

Namespace System

	Public Module ObjectX
		<Extension>
		Public Function ToJsonString(Of T As Class)(o As T, Optional unformattedLength As Integer = 90) As String
			Dim s = JsonConvert.SerializeObject(o, Formatting.None)
			Return If(s.Length < unformattedLength, s, JsonConvert.SerializeObject(o, Formatting.Indented))
		End Function

		''' <summary>trycast value to desired Type and assigns it.</summary>
		<DebuggerStepThrough(), Extension()>
		Public Function TryBe(Of T As Class)(ByRef target As T, value As Object) As Boolean
			Dim x = TryCast(value, T)
			If x Is Nothing Then Return False
			target = x
			Return True
		End Function
		''' <summary>cast value to desired Type and assigns it. Causes InvalidCastException on failure</summary>
		<DebuggerStepThrough(), Extension()>
		Public Sub Be(Of T)(ByRef target As T, value As Object)
			target = DirectCast(value, T)
		End Sub
		<DebuggerStepThrough(), Extension()>
		Public Function CastX(Of T)(Obj As Object) As T
			Return DirectCast(Obj, T)
		End Function
		<DebuggerStepThrough(), Extension()>
		Public Function TryCastX(Of T As Class)(Obj As Object) As T
			Return TryCast(Obj, T)
		End Function

		<DebuggerStepThrough(), Extension()>
		Public Function CloneX(Of T As ICloneable)(Obj As T) As T
			Return DirectCast(Obj.Clone(), T)
		End Function
		<Extension()>
		Public Function CloneSerial(Of T)(Obj As T) As T
			Using ms As New MemoryStream
				ms.Serialize(Obj)
				ms.Position = 0L
				Return ms.Deserialize(Of T)
			End Using
		End Function

		''' <summary>returns the subject itself. Useful in With-Blocks</summary>
		<DebuggerStepThrough(), Extension()>
		Public Function Self(Of T)(subj As T) As T
			Return subj
		End Function

		<Extension()>
		Public Function Exception(Of T)(sender As T, firstMsgSegment As Object, ParamArray msgSegments As Object()) As Exception(Of T)
			Return Exception(Of T)(sender, Nothing, firstMsgSegment, String.Concat(msgSegments))
		End Function

		<Extension()>
		Public Function Exception(Of T)(sender As T, innerException As Exception, ParamArray msgSegments As Object()) As Exception(Of T)
			Return New Exception(Of T)(sender, innerException, String.Concat(msgSegments))
		End Function

		<Extension()>
		Public Function Exception(Of T)(sender As T) As Exception(Of T)
			Return Exception(Of T)(sender, Nothing)
		End Function

		''' <summary> testet vor einer Zuweisung, ob der neue Wert überhaupt eine Änderung bringt </summary>
		''' <remarks>
		''' nützlich bei Zuweisungen an performance-intensive Properties, 
		''' oder wenn auf Änderungen reagiert werden muß
		''' </remarks>
		<Extension()>
		Public Function Assign(Of T)(ByRef Dest As T, Src As T) As Boolean
			If Object.Equals(Dest, Src) Then Return False
			Dest = Src
			Return True
		End Function

		<DebuggerStepThrough(), Extension()>
		Public Function Null(Of T As Structure)(Subj As T?) As Boolean
			Return Not Subj.HasValue
		End Function

		<DebuggerStepThrough(), Extension()>
		Public Function Null(Of T As Class)(Subj As T) As Boolean
			Return Subj Is Nothing
		End Function

		<DebuggerStepThrough(), Extension()>
		Public Function NotNull(Of T As Class)(Subj As T) As Boolean
			Return Subj IsNot Nothing
		End Function

	End Module
End Namespace
