﻿Option Compare Text

Imports System.Runtime.CompilerServices
Imports System.Text
Imports System.Collections
Imports System.Text.RegularExpressions
Imports System.Globalization
Imports System.ComponentModel

Namespace System

	Public Module StringX
		''' <summary>split the extended string to get entries. return empty array if string.IsNullOrWhiteSpace() </summary>
		<Extension()>
		Public Function GetEntries(sData As String, separator As String) As String()
			If String.IsNullOrWhiteSpace(sData) Then Return {}
			Return sData.Split(separator)
		End Function

		<Extension()>
		Public Sub AddLine(lines As List(Of String), ParamArray args As Object())
			lines.Add(String.Join(" ", args))
		End Sub

		Private _Counter, _MaxCount As Integer
		Private _patterns As Char() = "nt,:0123456789".ToCharArray
		Private Function FormatXEvaluater(mt As Match) As String
			Dim s = mt.Value
			Dim i = s.IndexOfAny(_patterns, 1, 1)
			If i > 0 Then 'einer der _patterns matcht
				Select Case Array.IndexOf(_patterns, s(i))
					Case 0 : Return Lf 'n matcht
					Case 1 : Return Tab 't matcht
					Case Is < 4 'einer der beiden FormatString-Segmentierer
					Case Else : Return s 'eine Ziffer
				End Select
			End If
			'_Counter als Ziffer einfügen
			If _Counter < _MaxCount Then _Counter += 1
			Return s.Insert(1, _Counter.ToString)
		End Function
		''' <summary> funzt wie String.Format, nur wird in leere '{}' laufende Nummern eingetragen - jedoch nicht höhere als Anzahl args. Sodass das letzte arg auch vielfach eingefügt werden kann </summary>
		<Extension()>
		Public Function FormatX(sFormat As String, ParamArray args As Object()) As String
			Return Format(CultureInfo.CurrentCulture, sFormat, args)
		End Function
		''' <summary> funzt wie String.Format, nur wird in leere '{}' laufende Nummern eingetragen - jedoch nicht höhere als Anzahl args. Sodass das letzte arg auch vielfach eingefügt werden kann </summary>
		<Extension()>
		Public Function FormatInvariant(sFormat As String, ParamArray args As Object()) As String
			Return Format(CultureInfo.InvariantCulture, sFormat, args)
		End Function
		Private Function Format(cult As CultureInfo, sFormat As String, ParamArray args As Object()) As String
			_Counter = -1 : _MaxCount = args.Length - 1
			Dim s = Regex.Replace(sFormat, "(?<=([^{]|^)({{)*){[^{}]*[}]", AddressOf FormatXEvaluater)
			Return String.Format(cult, s, args)
		End Function

		''' <summary> positive value cuts from left, negative cuts from right </summary>
		<Extension()>
		Public Function SubstringX(subj As String, cut As Integer) As String
			If cut < 0 Then Return subj.Substring(0, subj.Length + cut)
			Return subj.Substring(cut)
		End Function
		''' <summary> start and end instead of start and length. Adds negative numbers to subj.Length </summary>
		<Extension()>
		Public Function SubstringX(subj As String, start As Integer, [end] As Integer) As String
			If start < 0 Then start += subj.Length
			If [end] <= 0 Then [end] += subj.Length
			Return subj.Substring(start, [end] - start)
		End Function

		''' <summary> more comfortable than String.Split(separator as String(),count,mode) </summary>
		<Extension()>
		Public Function Split(subj As String, separator As String, count As Integer, Optional mode As StringSplitOptions = StringSplitOptions.None) As String()
			Return subj.Split(New String() {separator}, count, mode)
		End Function
		<Extension()>
		Public Function Replace(subj As String, oldValue As String, ParamArray concatNewValues() As Object) As String
			Return subj.Replace(oldValue, String.Concat(concatNewValues))
		End Function

		''' <summary> more comfortable to use than String.Split(separator as String(),mode) </summary>
		<Extension()>
		Public Function Split(subj As String, separator As String, Optional mode As StringSplitOptions = StringSplitOptions.None) As String()
			Return subj.Split(New String() {separator}, mode)
		End Function

		''' <summary> verkettet alles </summary>
		<Extension()>
		Public Sub [Set](ByRef subj As String, ParamArray others() As Object)
			subj = String.Concat(others)
		End Sub

		<EditorBrowsable(EditorBrowsableState.Never)>
		Public Function ReplaceBslashN(s As String) As String
			Dim sb As New StringBuilder
			For i = 0 To s.Length - 2 Step 2
				If s.IndexOf("\n", i, 2) < 0 Then Exit For
				sb.Append(CrLf)
			Next
			sb.Append(s.Substring(sb.Length))
			Return sb.ToString
		End Function

		'''<summary> verkettet alles, interpretiert \n am Anfang als newline</summary>
		<Extension()>
		Public Function And2(subj As String, ParamArray others() As Object) As String
			Dim sb = New StringBuilder(ReplaceBslashN(subj))
			For Each itm In others
				sb.Append(ReplaceBslashN(itm.ToString))
			Next
			Return sb.ToString
		End Function

		'''<summary> verkettet alles</summary>
		<Extension()>
		Public Function [And](subj As String, ParamArray others() As Object) As String
			Return String.Concat(subj, String.Concat(others))
		End Function

		''' <returns>den Stringabschnitt ab dem letzten Auftreten von sMatch - optional einschließlich des Matches</returns>
		<Extension()>
		Public Function LastRightCut(s As String, sMatch As String, Optional includePattern As Boolean = False) As String
			Dim i = s.LastIndexOf(sMatch)
			If i < 0 Then Return Nothing
			Return s.Substring(If(includePattern, i, i + sMatch.Length))
		End Function

		''' <returns>den Stringabschnitt ab dem ersten Auftreten von sMatch - optional einschließlich des Matches</returns>
		<Extension()>
		Public Function RightCut(S$, sMatch$, Optional includePattern As Boolean = False) As String
			Dim I% = S.IndexOf(sMatch)
			If I < 0 Then Return Nothing
			Return S.Substring(If(includePattern, I, I + sMatch.Length))
		End Function

		Public Function CreateUnescapeRegex(pattern As String) As Regex
			Return New Regex("(?<=(^|[^\\](\\\\)*))" & pattern) 'der vorangestellte pattern schließt escaped Matches ('\X' etc.) aus
		End Function

		''' <summary>gibt den String-Abschnitt links des letzten gefundenen Matches zurück - optional einschließlich des Matches</summary>
		<Extension()>
		Public Function LastLeftCut(subj As String, pattern As String, Optional includePattern As Boolean = False) As String
			Dim i = subj.LastIndexOf(pattern)
			Return If(i >= 0, subj.Substring(0, If(includePattern, i + pattern.Length, i)), Nothing)
		End Function

		''' <summary>sucht von rechts count Auftreten von pattern, returnt den Text links davon</summary>
		<Extension()>
		Public Function LastLeftCut(subj As String, pattern As String, matchSkips As Integer) As String
			Dim start = subj.Length
			For ii = 0 To matchSkips
				start = subj.LastIndexOf(pattern, start) - 1
				If start < -1 Then Return Nothing
			Next
			Return subj.Substring(0, start + 1)
		End Function

		''' <summary> gibt den String-Abschnitt links des ersten gefundenen Matches zurück - optional einschließlich des Matches</summary>
		<Extension()>
		Public Function LeftCut(subj As String, pattern As String, Optional includePattern As Boolean = False) As String
			Dim i = subj.IndexOf(pattern)
			Return If(i >= 0, subj.Substring(0, If(includePattern, i + pattern.Length, i)), Nothing)
		End Function

		Public Enum Additional : None : Start : [End] : Start_End : End Enum
		<Extension()>
		Public Function Between(separator As Char, additional As Additional, ParamArray args() As Object) As String
			Return Between(args, separator, additional)
		End Function
		<Extension()>
		Public Function Between(separator As Char, ParamArray args() As Object) As String
			Return Between(args, separator, Additional.None)
		End Function
		<Extension()>
		Public Function Between(separator As String, additional As Additional, ParamArray args() As Object) As String
			Return Between(args, separator, additional)
		End Function
		<Extension()>
		Public Function Between(separator As String, ParamArray args() As Object) As String
			If args.Length = 1 Then
				'special: args is just 2 chars in a string - treat the chars as opening and closing bracket or Quotes
				Dim s As String = TryCast(args(0), String)
				If s IsNot Nothing AndAlso s.Length = 2 Then Return String.Concat(s(0), separator, s(1))
			End If
			Return Between(args, separator, Additional.None)
		End Function

		Private _SB As New System.Text.StringBuilder
		Private Function Between(args() As Object, separator As String, additional As Additional) As String
			While args.Length = 1 AndAlso Not TypeOf args(0) Is String
				Dim a = TryCast(args(0), IEnumerable)
				If a Is Nothing Then Exit While
				args = a.Cast(Of Object).ToArray
			End While
			_SB.Clear()
			If additional.HasFlag(Additional.Start) Then _SB.Append(separator)
			_SB.Append(String.Join(separator, args))
			If additional.HasFlag(Additional.End) Then _SB.Append(separator)
			Return _SB.ToString
		End Function

		<Extension()>
		Public Function CountChars(S As String, Search As Char) As Integer
			CountChars = 0
			Dim I As Integer = S.IndexOf(Search)
			While I >= 0
				CountChars += 1
				I = S.IndexOf(Search, I + 1)
			End While
		End Function

		<Extension()>
		Public Function CountSubStrings(S As String, Search As String,
				Optional Comparison As StringComparison = StringComparison.CurrentCulture) As Integer
			CountSubStrings = 0
			Dim I As Integer = S.IndexOf(Search, 0, Comparison)
			While I >= 0
				CountSubStrings += 1
				I = S.IndexOf(Search, I + 1, Comparison)
			End While
		End Function

		<Extension()>
		Public Function TryParseX(s As String, defaultValue As Integer) As Integer
			Dim val As Integer : Return If(Integer.TryParse(s, val), val, defaultValue)
		End Function
		<Extension()>
		Public Function TryParseX(s As String, defaultValue As Double) As Double
			Dim val As Double : Return If(Double.TryParse(s, val), val, defaultValue)
		End Function
		<Extension()>
		Public Function TryParseX(s As String, defaultValue As Date) As Date
			Dim val As Date : Return If(Date.TryParse(s, val), val, defaultValue)
		End Function
		<Extension()>
		Public Function TryParseX(s As String, defaultValue As TimeSpan) As TimeSpan
			Dim val As TimeSpan : Return If(TimeSpan.TryParse(s, val), val, defaultValue)
		End Function

	End Module


End Namespace