﻿Imports System.Runtime.CompilerServices

Public Module DateTimeX

	Public Enum TimeUnit : Tick : Millisecond : Second : Minute : Hour : Day : End Enum
	Public _TicksPerUnit As Long() = {1, TimeSpan.TicksPerMillisecond, TimeSpan.TicksPerSecond, TimeSpan.TicksPerMinute, TimeSpan.TicksPerHour, TimeSpan.TicksPerDay}

	<Extension>
	Public Function Align(dt As Date, unit As TimeUnit) As Date
		Dim rest As Long = 0
		Dim ticks = Math.DivRem(dt.Ticks, _TicksPerUnit(unit), rest)
		Return dt.AddTicks(-rest)
	End Function

	Sub TestAlign()
		Dim dt0 = Date.Now
		Dim dt1 = dt0.AddTicks(123456)
		Dim dt2 = Align(dt1, TimeUnit.Second)
		Dim ts0 = dt0.TimeOfDay
		Dim ts1 = dt1.TimeOfDay
		Dim ts2 = dt2.TimeOfDay
	End Sub ' <-- am Haltepunkt die Timespans vergleichen


	Public Enum Holidays
		None = 0
		Neujahr = 1 + (1 << 16)
		Maifeiertag = 1 + (5 << 16)
		TagDerDeutschenEinheit = 3 + (10 << 16)
		Heiligabend = 24 + (12 << 16)
		ErsterWeihnachtsfeiertag = 25 + (12 << 16)
		ZweiterWeihnachtsfeiertag = 26 + (12 << 16)
		Silvester = 31 + (12 << 16)
		Karfreitag = 2
		Ostermontag
		ChristiHimmelfahrt
		Pfingstmontag
		Fronleichnam
	End Enum
	Private _HolyDays As New HashSet(Of Integer)(DirectCast([Enum].GetValues(GetType(Holidays)), Integer()))
	<Extension()>
	Public Function GetHoliday(dtDate As DateTime) As Holidays
		dtDate = dtDate.Date
		Dim n = dtDate.Day + (dtDate.Month << 16)
		If _HolyDays.Contains(n) Then Return DirectCast(n, Holidays)
		Dim Easter As DateTime = GetEaster(dtDate.Year)
		If dtDate = Easter.AddDays(-2) Then Return Holidays.Karfreitag
		If dtDate = Easter.AddDays(1) Then Return Holidays.Ostermontag
		If dtDate = Easter.AddDays(39) Then Return Holidays.ChristiHimmelfahrt
		If dtDate = Easter.AddDays(50) Then Return Holidays.Pfingstmontag
		If dtDate = Easter.AddDays(60) Then Return Holidays.Fronleichnam
		Return Holidays.None
	End Function
	''' <summary>gibt die Anzahl der Arbeitstage zwischen Start und End</summary>
	<Extension()>
	Public Function GetWorkDays(DateStart As Date, DateEnd As Date) As Integer
		If DateStart >= DateEnd Then DateStart = DateStart.AddMonths(-1)
		Dim Days As Integer = DateEnd.Subtract(DateStart).Days + 1
		For i As Integer = 0 To Days - 1
			If Not DateStart.AddDays(i).GetHoliday = Holidays.None OrElse DateStart.AddDays(i).DayOfWeek = DayOfWeek.Saturday OrElse DateStart.AddDays(i).DayOfWeek = DayOfWeek.Sunday Then
				Days -= 1
			End If
		Next
		Return Days
	End Function
	''' <summary>Returnt den letzten bis einschl. dToday zu arbeitenden Tag (berücksichtigt derzeit keine Feiertage)</summary>
	<Extension()>
	Public Function GetCapDate(dToday As Date) As Date
		Select Case dToday.DayOfWeek
			Case DayOfWeek.Saturday
				dToday = dToday.AddDays(-1)
			Case DayOfWeek.Sunday
				dToday = dToday.AddDays(-2)
			Case Else
		End Select
		Return dToday
	End Function

	Private Function GetEaster(year As Integer) As DateTime
		Dim a As Integer = year Mod 19
		Dim b As Integer = year \ 100
		Dim c As Integer = (8 * b + 13) \ 25 - 2
		Dim d As Integer = b - (year \ 400) - 2
		Dim e As Integer = (19 * (year Mod 19) + ((15 - c + d) Mod 30)) Mod 30
		If e = 28 Then
			If a > 10 Then
				e = 27
			End If
		ElseIf e = 29 Then
			e = 28
		End If
		Dim f As Integer = (d + 6 * e + 2 * (year Mod 4) + 4 * (year Mod 7) + 6) Mod 7
		Return New DateTime(year, 3, 22).AddDays(e + f)
	End Function

End Module
