﻿Imports System.Runtime.CompilerServices
Imports System.Reflection
Imports System.ComponentModel
Imports System.Data
Imports System.IO

Namespace System.Windows.Forms
   Public Module XDatasetAdapter
      <Extension()> _
      Public Function SuspendConstraints(dts As DataSet) As IDisposable
         Return dts.Attach.SuspendConstraints.Enter
      End Function
      <Extension()> _
      Public Sub RegisterSaveCallback(dts As DataSet, dlg As Action, Optional value As Boolean = True)
         dts.Attach.RegisterSaveCallback(dlg, value)
      End Sub
      <Extension()> _
      Public Function Attach(dts As DataSet) As DataTableAttach.DatasetAttach
         Return dts.Tables(0).Attach.Dataset_Attach
      End Function
      <Extension()> _
      Public Function Adapter(dts As DataSet) As DatasetAdapterBase
         Adapter = Nothing
         If DatasetAdapterBase.Attachs.TryGetValue(dts, Adapter) Then Exit Function
         Adapter = New DatasetXmlAdapter(dts)
         DatasetAdapterBase.Attachs.Add(dts, Adapter)
      End Function
      <Extension()>
      Public Function DataFile(Of T As DataSet)(dts As T, path As String) As T
         DirectCast(dts.Adapter, DatasetXmlAdapter).DataFile = New FileInfo(path)
         Return dts
      End Function
      <Extension()> _
      Public Function DataFile(dts As DataSet) As FileInfo
         Dim att = DirectCast(dts.Adapter, DatasetXmlAdapter)
         If att.Null Then Return Nothing
         Return att.DataFile
      End Function

      ''' <summary>Clear dataset and refill. Optional only the given tables (not supported in dataset-only-scenarios)</summary>
      <Extension()> _
      Public Sub Fill(dts As DataSet, ParamArray tables() As DataTable)
         dts.Adapter.Fill(tables)
      End Sub
      <Extension()> _
      Public Sub Save(dts As DataSet, frm As Form, Optional checkForChanges As Boolean = True)
         dts.Adapter.Save(frm, checkForChanges)
      End Sub

      <Extension()> _
      Public Function HandleFormClosing(dts As DataSet) As FormClosingEventHandler
         Return dts.Adapter.HandleFormClosing
      End Function

#Region "Winfom-Close-Bug"         ' Aufwändig gestaltete DGVs schmeißen manchmal Fehler beim Disposen des Forms.

      Private Sub Form_Closed(s As Object, e As EventArgs)
         'manchmal schmeissen DGVs Fehler beim Schließen des Forms. Ist schlecht reproduzierbar.
         'Reproduziere: Starte RaslDb, gehe auf Artikel-Tab und Form schließen
         With DirectCast(s, Form)
            If .DialogResult <> DialogResult.None Then Return
            .Validate() 'nochn Bug: DGV kriegt beim Disposen einen ZeilenIndex-Fehler, wenns sich die ZufügeZeile im Edit-Modus befindet
            .SuspendLayout()
#If True Then            'geringster Eingriff
            For Each ctl In New GetChilds(Of Control)(Function(c) c.Controls).AllAsList(.Self)
               With TryCast(ctl, DataGridView)
                  If .NotNull Then .Parent.SuspendLayout() : .Dispose()
               End With
            Next
#ElseIf True Then            'einfachster Eingriff
            With .Controls
               'Controls könnten sich theor. auch gegenseitig disposen, über Events. dieses hier disposed ganz sicher immer das letzte Element was noch da ist
               While .Count > 0 : .Item(.Count - 1).Dispose() : End While
            End With
#ElseIf True Then            'kein Eingriff
         Return
#ElseIf True Then
#End If
         End With
      End Sub

      'Beim Schließen des MainForms wird Form_Closed der anneren Forms garnet mehr aufgerufen. Daher hier alle alle Controls disposen
      Private Sub MainForm_Closed(s As Object, e As EventArgs)
         For Each frm As Form In Application.OpenForms
            Form_Closed(frm, e)
         Next
      End Sub

#End Region 'Winfom-Close-Bug

      Friend _KnownDatasets As New Dictionary(Of Type, DataSet)

      Private _FormClosedKey As Object = GetType(Form) _
         .GetField("EVENT_FORMCLOSED", BindingFlags.NonPublic Or BindingFlags.Static) _
         .GetValue(Nothing)

      Private Sub ThrowIfFormIsAlreadyRegistered(frm As Form)
         ' mit Reflection den Form_Closed-Delegaten auspuhlen, und gucken, ob 'mein' Form_Closed - Handler bereits eingetragen ist. Falls ja - Exception
         Dim events = DirectCast(DataTableAttach.EventsFieldInfo.GetValue(frm), EventHandlerList)
         Dim formClosed = events(_FormClosedKey)
         With events(_FormClosedKey)
            If .NotNull AndAlso .GetInvocationList.Contains(DirectCast(AddressOf Form_Closed, FormClosedEventHandler)) Then
               Throw New InvalidOperationException("can't register the same Form twice!")
            End If
         End With
      End Sub

#If True Then
      ''' <summary>
      ''' registers all datasets of this type and all bindingsources of the form, to improve performance and to avoid data-redundance
      ''' </summary>
      <Extension()> _
      Public Function Register(Of T As DataSet)(dts As T, frm As Form, handleFormClosing As Boolean) As T
         ThrowIfFormIsAlreadyRegistered(frm)
         AddHandler frm.FormClosed, AddressOf Form_Closed 'EVENT_FORMCLOSED
         If handleFormClosing Then AddHandler frm.FormClosing, dts.Adapter.HandleFormClosing
         Return Register(frm, dts)
      End Function

      ''' <summary>
      ''' registers all datasets of this type and all bindingsources of the ContainerControl, to improve performance and to avoid data-redundance
      ''' </summary>
      <Extension()> _
      Public Function Register(Of T As DataSet)(dts As T, ctlContainer As ContainerControl) As T
         Dim frm = TryCast(ctlContainer, Form)
         If frm.NotNull Then Return dts.Register(frm, False)
         Return Register(ctlContainer, dts)
      End Function

      Private Function Register(Of T As DataSet)(frm As ContainerControl, dts As T) As T
         Dim tpDts = dts.GetType
         Dim tmp As DataSet = Nothing
         Dim wasKnown = _KnownDatasets.TryGetValue(tpDts, tmp)
         If wasKnown Then dts = DirectCast(tmp, T)
         Register = dts
         Dim allCtls = New GetChilds(Of Control)(Function(ctl) ctl.Controls).AllAsList(frm)
         Dim dgvSuspend = New GridSuspend(dts, allCtls)
         Dim oldDatasets As New HashSet(Of DataSet)
         Dim shouldReplace As Func(Of Object, Boolean) = Function(src) _
            src IsNot Nothing AndAlso _
            (src Is tpDts OrElse src.GetType Is tpDts AndAlso src IsNot dts)
         Dim bindFlags As BindingFlags = BindingFlags.Instance Or BindingFlags.Public _
            Or BindingFlags.NonPublic Or BindingFlags.GetField
         For Each ctl In allCtls.OfType(Of ContainerControl)()
            Dim fields = ctl.GetType.GetFields(bindFlags)
            For Each fld In fields
               If fld.FieldType Is tpDts Then
                  oldDatasets.Add(DirectCast(fld.GetValue(ctl), DataSet))
                  fld.SetValue(ctl, dts)
                  Continue For
               End If
               If Not GetType(IComponent).IsAssignableFrom(fld.FieldType) Then Continue For
               Dim prp = fld.FieldType.GetProperty("DataSource")
               If prp.Null Then Continue For
               Dim itm = fld.GetValue(ctl)
               If itm Is Nothing Then Continue For
               TryRegisterAsBindingSource(itm, dts, tpDts)
               If shouldReplace(prp.GetValue(itm, Nothing)) Then prp.SetValue(itm, dts, Nothing)
            Next
         Next
         dgvSuspend.Resume()
         oldDatasets.Remove(dts)
         For Each ds In oldDatasets : ds.Dispose() : Next
         If Not wasKnown Then _KnownDatasets.Add(tpDts, dts)
      End Function
#ElseIf True Then
      ''' <summary>
      ''' registers all datasets of this type and all bindingsources of the form, to improve performance and to avoid data-redundance
      ''' </summary>
      <Extension()> _
      Public Function Register(Of T As DataSet)(dts As T, frm As Form, handleFormClosing As Boolean) As T
         ThrowIfFormIsAlreadyRegistered(frm)
         Dim tpDts = dts.GetType
         Dim tmp As DataSet = Nothing
         Dim wasKnown = _KnownDatasets.TryGetValue(tpDts, tmp)
         If wasKnown Then dts = DirectCast(tmp, T)
         Register = dts
         AddHandler frm.FormClosed, AddressOf Form_Closed 'EVENT_FORMCLOSED
         If handleFormClosing Then AddHandler frm.FormClosing, dts.Adapter.HandleFormClosing
         Dim allCtls = New GetChilds(Of Control)(Function(ctl) ctl.Controls).AllAsList(frm)
         Dim dgvSuspend = New GridSuspend(dts, allCtls)
         Dim oldDatasets As New HashSet(Of DataSet)
         Dim shouldReplace As Func(Of Object, Boolean) = Function(src) _
            src IsNot Nothing AndAlso _
            (src Is tpDts OrElse src.GetType Is tpDts AndAlso src IsNot dts)
         Dim bindFlags As BindingFlags = BindingFlags.Instance Or BindingFlags.Public _
            Or BindingFlags.NonPublic Or BindingFlags.GetField
         For Each ctl In allCtls.OfType(Of ContainerControl)()
            Dim fields = ctl.GetType.GetFields(bindFlags)
            For Each fld In fields
               If fld.FieldType Is tpDts Then
                  oldDatasets.Add(DirectCast(fld.GetValue(ctl), DataSet))
                  fld.SetValue(ctl, dts)
                  Continue For
               End If
               If Not GetType(IComponent).IsAssignableFrom(fld.FieldType) Then Continue For
               Dim prp = fld.FieldType.GetProperty("DataSource")
               If prp.Null Then Continue For
               Dim itm = fld.GetValue(ctl)
               If itm Is Nothing Then Continue For
               TryRegisterAsBindingSource(itm, dts, tpDts)
               If shouldReplace(prp.GetValue(itm, Nothing)) Then prp.SetValue(itm, dts, Nothing)
            Next
         Next
         dgvSuspend.Resume()
         oldDatasets.Remove(dts)
         For Each ds In oldDatasets : ds.Dispose() : Next
         If Not wasKnown Then _KnownDatasets.Add(tpDts, dts)
      End Function
#ElseIf True Then
#End If


      Private Sub TryRegisterAsBindingSource(itm As Object, dts As DataSet, tpDts As Type)
         Dim bs = TryCast(itm, BindingSource)
         If bs.Null Then Return
         Dim dataMember = bs.DataMember
         Dim dataSource = bs.DataSource
         If dataSource Is Nothing Then Return
         Do
            With TryCast(dataSource, BindingSource)
               If .Null Then Exit Do
               If dataMember = "" Then dataMember = .DataMember
               dataSource = .DataSource
            End With
         Loop
         If dataSource IsNot tpDts AndAlso dataSource.GetType IsNot tpDts Then Return
         With dts.Tables(dataMember)
            If .NotNull Then .Attach.AddSource(bs) : Return
         End With
         With dts.Relations(dataMember)
            If .NotNull Then .ChildTable.Attach.AddSource(bs)
         End With
      End Sub

      Private Class GridSuspend
         'beim Umstöpseln der BindingSources die DGVs abhängen
         Private _Tuples As New List(Of Tuple(Of DataGridView, BindingSource))

         Public Sub New(dts As DataSet, allCtls As List(Of Control))
            If dts.Tables.Cast(Of DataTable).All(Function(tb) tb.Rows.Count = 0) Then Return
            For Each dgv In allCtls.OfType(Of DataGridView)()
               Dim bs = TryCast(dgv.DataSource, BindingSource)
               If bs.Null Then Continue For
               _Tuples.Add(Tuple.Create(dgv, bs))
               dgv.DataSource = Nothing
            Next
         End Sub
         Public Sub [Resume]()
            For Each tpl In _Tuples
               tpl.Item1.DataSource = tpl.Item2
            Next
         End Sub
      End Class

   End Module
End Namespace

