Imports System.ComponentModel
Imports System.Collections

Namespace System.Windows.Forms

   ''' <summary> kann zur Designzeit Meldungen ausgeben (hnlich Direktfenster). </summary>
   <System.ComponentModel.DesignerCategory("Code")> _
   Public Class DesignDebugger : Inherits Control

      Private Shared Event MsgEvent As EventHandler
      Private Shared _Texts As New List(Of Object)

      Private ReadOnly _StringFormat As StringFormat
      Private _LayoutRect As Rectangle
      Private _Clear As Boolean
      Private _ToolTip As New ToolTip

      Public Sub New()
         Me.BackColor = System.Drawing.SystemColors.Window
         ''Me.Width = 500
         Me.Dock = DockStyle.Left
         MyBase.TabStop = False
         _StringFormat = New StringFormat
         _StringFormat.LineAlignment = StringAlignment.Far
         AddHandler MsgEvent, AddressOf DesignDebugPanel_Msg
         _ToolTip.SetToolTip(Me, "DoubleClick clears Output")
      End Sub

      Public Sub DesignDebugPanel_Msg(sender As Object, e As EventArgs)
         Me.Invalidate()
      End Sub

      ''' <summary> _LayoutRect neu festlegen </summary>
      Protected Overrides Sub OnResize(eventargs As System.EventArgs)
         MyBase.OnResize(eventargs)
         _LayoutRect = ClientRectangle
         _LayoutRect.Y = -(Short.MaxValue - _LayoutRect.Height)
         _LayoutRect.Height = Short.MaxValue
         Invalidate()
      End Sub

      Protected Overrides Sub OnPaint(e As System.Windows.Forms.PaintEventArgs)
         MyBase.OnPaint(e)
         e.Graphics.DrawString( _
          Lf.Between(_Texts), Me.Font, Brushes.Black, _LayoutRect, _StringFormat)
      End Sub

      ''' <summary>Melde-Zeile ausgeben</summary>
      Public Shared Sub Msg(ParamArray Args() As Object)
         Msg(Args, Nothing)
      End Sub
      Public Shared Sub Msg(Arg1 As String, Arg2 As String)
         Msg(New String() {Arg1, Arg2})
      End Sub
      ''' <summary>Melde-Zeile ausgeben, format wird auf jedes arg angewandt</summary>
      Public Shared Sub Msg(Args As IEnumerable, Optional format As String = Nothing)
         format = "{0" & If(format.Null, "}", ",".And(format, "}"))
         Dim texts = New List(Of String)
         For Each Arg As Object In Args
            texts.Add(If(Arg Is Nothing, "NULL", String.Format(format, Arg)))
         Next
         Msg(" ".Between(texts))
      End Sub
      ''' <summary>Meldung ausgeben</summary>
      Public Shared Sub Msg(sMsg As String)
         If _Texts.Count > 100 Then _Texts.RemoveAt(0)
         Debug.WriteLine(sMsg)
         _Texts.Add(sMsg)
         RaiseEvent MsgEvent(Nothing, EventArgs.Empty)
      End Sub

      ''' <summary> Doppelklick im Designer auf diese Property lscht die Meldungen </summary>
      ''' <remarks>Der Wert bleibt auf False</remarks>
      <Browsable(True), Category("Verhalten")> _
      Public Property Clear() As Boolean
         Get
            Return _Clear
         End Get
         Set(NewValue As Boolean)
            _Texts.Clear()
            Me.Invalidate()
         End Set
      End Property

      <DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)> _
      <Browsable(False)> _
      Public Shadows ReadOnly Property TabStop() As Boolean
         Get
            Return False
         End Get
      End Property

      Private Sub DesignDebugger_DoubleClick(sender As Object, e As System.EventArgs) _
            Handles Me.DoubleClick
         _Texts.Clear()
         Me.Invalidate()
      End Sub
   End Class 'DesignDebugger

End Namespace