﻿#Region "FileHeader"
#If False Then
ReInitialize() ist ziemlich hacky. Das generiert für DgvComboColumns nämlich unsichtbare ExpressionColumns an die DataTable dran, zum Werte-Abruf übergeordneter DataTables.
Wenn man dann einen Filter oder eine Sortierung entsprechend des DGVColumn-HeaderTextes festlegt, dann wird nicht der DgvColumn.DatapropertyName genommen, sondern die ExpressionColumn. So sortiert es textuell, und man kann mit Text-Vergleich-Filtern arbeiten, auch wenn die dgvColumn ein Integer-FK ist.
#End If '-- Options, Imports
Imports System.ComponentModel
Imports System.Text.RegularExpressions
#End Region 'FileHeader
Namespace System.Windows.Forms
	Public Class uclCheckedText

		Public Enum Mode : Sort : Filter : Filter_NoDisplayOnStringEmpty : End Enum

		Private WithEvents _Dgv As DataGridView
		Private _rgxWord As New Regex("\b\w+\b", RegexOptions.IgnoreCase)
		Private _dicColInfos As New Dictionary(Of String, Tuple(Of DataGridViewColumn, String)) ' map HeaderText to Sort-/Filter-PropertyName

		Public Sub New()
			InitializeComponent()
			If NotifyPropertyChanged.IsInDesignMode Then Return
			Dim x = New WordSelectBehavior(txtText)
			AddHandler ckApply.CheckStateChanged, AddressOf Checkbox_CheckStateChanged
			AddHandler txtText.TextChanged, AddressOf txtFilter_TextChanged
		End Sub

		Private Sub txtFilter_TextChanged(sender As Object, e As EventArgs)
			'wenn der Text changed, geht die zugehörige Checkbox auf Indeterminate
			Dim ck = ckApply
			If ck.Checked Then ck.CheckState = CheckState.Indeterminate
		End Sub

		Private Sub ckApplyFilter_Click(sender As Object, e As EventArgs) Handles ckApply.Click
			'die Logik, mit die hiesigen 3-wertige Checkboxen auf Klick reagieren - beachte, dass sie ausserdem auf Textbox-TextChanges reagieren
			Dim ck = DirectCast(sender, CheckBox)
			Select Case ck.CheckState
				Case CheckState.Checked : ck.CheckState = CheckState.Unchecked
				Case CheckState.Unchecked, CheckState.Indeterminate : ck.CheckState = CheckState.Checked
			End Select
		End Sub

		Private Sub Checkbox_CheckStateChanged(sender As Object, e As EventArgs)
			ApplyChoice()
		End Sub

		Public Sub ApplyChoice()
			'wenn die Checkbox gecheckt, wird ApplyAction ausgeführt, wenn geuncheckt abgeschaltet, bei Indeterminate bleibts wies war
			Select Case ckApply.CheckState
				Case CheckState.Unchecked : ApplyAction("")
				Case CheckState.Checked
					Try
						ApplyAction(txtText.Text)
					Catch ex As Exception
						Msg(ex.Message)
						ckApply.CheckState = CheckState.Indeterminate
					End Try
				Case CheckState.Indeterminate
			End Select
		End Sub
		Private Sub ApplyAction(s As String)
			Dim bs = DirectCast(_Dgv.DataSource, BindingSource)
			If s = "" Then
				Select Case ActionMode
					Case Mode.Sort : bs.Sort = "" : _Dgv.Attach.DrawGroupLines(False)
					Case Mode.Filter : bs.Filter = ""
					Case Mode.Filter_NoDisplayOnStringEmpty : bs.Filter = "1 = 2"
					Case Else : Throw New InvalidOperationException("unknown ActionMode")
				End Select
				Return
			End If
			Dim sortCols As New List(Of DataGridViewColumn)
			Dim evaluate As MatchEvaluator = Function(mt)
															Dim tpl As Tuple(Of DataGridViewColumn, String) = Nothing
															If _dicColInfos.TryGetValue(mt.Value, tpl) Then
																sortCols.Add(tpl.Item1)
																Return tpl.Item2
															End If
															Return mt.Value
														End Function
			s = _rgxWord.Replace(s, evaluate)
			Select Case ActionMode
				Case Mode.Sort
					bs.Sort = s
					For i = 0 To sortCols.Count - 1
						sortCols(i).DisplayIndex = i
					Next
					_Dgv.Attach.DrawGroupLines(sortCols.ToArray())

				Case Mode.Filter, Mode.Filter_NoDisplayOnStringEmpty : bs.Filter = s
				Case Else : Throw New InvalidOperationException("unknown ActionMode")
			End Select
		End Sub

		Private Sub _Dgv_VisibleChanged(sender As Object, e As EventArgs)
			'detect Form_Shown
			If _Dgv.Visible Then
				RemoveHandler _Dgv.VisibleChanged, AddressOf _Dgv_VisibleChanged
				ReInitialize()
			End If
		End Sub

		Public Sub ReInitialize()
			_ContextMenu.Items.Clear()
			Dim cols = (From col In _Dgv.Columns.Cast(Of DataGridViewColumn) Where col.DataPropertyName <> "" Order By col.HeaderText).ToArray
			_ContextMenu.Items.AddRange(cols.Select(Function(x) New ToolStripMenuItem(x.HeaderText)).ToArray)
			_dicColInfos = cols.ToDictionary(Function(x) x.HeaderText, Function(x) Tuple.Create(x, x.DataPropertyName))
			Dim tb = DirectCast(_Dgv.DataSource, BindingSource).DataTable
			Dim prntRels = tb.ParentRelations.Cast(Of DataRelation).ToArray
			For Each col In cols.OfType(Of DataGridViewComboBoxColumn)
				Dim tbCol = tb.Columns(col.DataPropertyName)
				If tbCol.DataType Is GetType(String) Then Continue For
				Dim newName = $"{col.DataPropertyName}_{col.DisplayMember}"
				_dicColInfos(col.HeaderText) = New Tuple(Of DataGridViewColumn, String)(col, newName)
				If tb.Columns.Contains(newName) Then Continue For
				Dim rl = prntRels.FirstOrDefault(Function(x) x.ChildColumns(0) Is tbCol)
				Dim expr = $"Parent({rl.RelationName}).{col.DisplayMember}"
				Dim newCol = tb.Columns.Add(newName, GetType(String), expr)
			Next
			ApplyChoice()
		End Sub

		<Category("Behaviour"), DefaultValue(Mode.Sort)>
		Public Property ActionMode() As Mode

		<Category("Behaviour"), DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)>
		Public Property DatagridView() As DataGridView
			Get
				Return _Dgv
			End Get
			Set(ByVal value As DataGridView)
				If _Dgv Is value Then Return
				_Dgv = value
				If Not NotifyPropertyChanged.IsInDesignMode Then AddHandler _Dgv.VisibleChanged, AddressOf _Dgv_VisibleChanged
			End Set
		End Property

		<Browsable(True), DesignerSerializationVisibility(DesignerSerializationVisibility.Visible), DefaultValue("")>
		Public Overrides Property Text As String
			Get
				Return txtText.Text
			End Get
			Set
				txtText.Text = Value
			End Set
		End Property

		<Category("Appearance"), DefaultValue(CheckState.Unchecked)>
		Public Property Checkstate As CheckState
			Get
				Return ckApply.CheckState
			End Get
			Set
				ckApply.CheckState = Value
			End Set
		End Property

		<Category("Appearance"), DefaultValue("Apply")>
		Public Property Caption As String
			Get
				Return ckApply.Text
			End Get
			Set
				ckApply.Text = Value
			End Set
		End Property

		<Category("Appearance"), DefaultValue(True)>
		Public Property MultiLine As Boolean
			Get
				Return txtText.Multiline
			End Get
			Set
				txtText.Multiline = Value
			End Set
		End Property

		Private Sub _ContextMenu_ItemClicked(sender As Object, e As ToolStripItemClickedEventArgs) Handles _ContextMenu.ItemClicked
			Dim i = txtText.SelectionStart
			Dim spc = If(i > 0 AndAlso txtText.Text(i - 1) = " ", "", " ")
			txtText.SelectedText = spc & e.ClickedItem.Text
		End Sub
	End Class

End Namespace
