﻿Imports System.Runtime.CompilerServices

Namespace System.Windows.Forms

   Public Module ControlX
#Region "DataGridView"
      ''' <summary> enumeriert selectierten DataRowViews </summary>
      <Extension()> _
      Public Function SelectedDrvs(dgv As DataGridView) As IEnumerable(Of DataRowView)
         Dim x = From rwDgv In dgv.SelectedCells.Cast(Of DataGridViewCell)() Select rwDgv.OwningRow Distinct Select OwningRow.DataBoundItem
         Return x.OfType(Of DataRowView)()
      End Function
      ''' <summary> enumeriert selectierten DataRowViews </summary>
      <Extension()> _
      Public Function SelectedDataRows(Of T As DataRow)(dgv As DataGridView) As IEnumerable(Of T)
         Dim x = From rwDgv In dgv.SelectedCells.Cast(Of DataGridViewCell)() Select rwDgv.OwningRow Distinct Select OwningRow.DataBoundItem
         Return x.OfType(Of DataRowView)().Select(Function(drv) DirectCast(drv.Row, T))
      End Function
      ''' <summary> enumeriert die DataRowViews, bei denen die CheckboxCell gecheckt ist </summary>
      <Extension()> _
      Public Iterator Function CheckedDrvs(col As DataGridViewCheckBoxColumn) As IEnumerable(Of DataRowView)
         Dim dgv = col.DataGridView
         Dim colIndx = col.Index
         For i = 0 To dgv.RowCount - 1 - dgv.AllowUserToAddRows.AsIndex
            Dim rwDgv = dgv.Rows(i)
            If CBool(rwDgv.Cells(colIndx).EditedFormattedValue) Then Yield DirectCast(rwDgv.DataBoundItem, DataRowView)
         Next
      End Function
      ''' <summary> enumeriert die DataRows, bei denen die CheckboxCell gecheckt ist </summary>
      <Extension()> _
      Public Function CheckedDataRows(Of T As DataRow)(col As DataGridViewCheckBoxColumn) As IEnumerable(Of T)
         Return From drv In col.CheckedDrvs Select DirectCast(drv.Row, T)
      End Function
      <Extension()> _
      Public Sub SetValue(col As DataGridViewColumn, rowIndex As Integer, value As Object)
         col.DataGridView(col.Index, rowIndex).Value = value
      End Sub
      ''' <summary> returns null if invalid rowIndex </summary>
      <Extension()> _
      Public Function DataRow(Of T As DataRow)(dgv As DataGridView, rowIndex As Integer) As T
         If rowIndex < 0 Then Return Nothing
         Return dgv.Rows(rowIndex).DataRow(Of T)()
      End Function
      <Extension()> _
      Public Function DataRow(Of T As DataRow)(dgvRow As DataGridViewRow) As T
         Dim itm = DirectCast(dgvRow.DataBoundItem, DataRowView)
         Return If(itm.Null, Nothing, DirectCast(itm.Row, T))
      End Function
		''' <summary> returns null if invalid rowIndex </summary>
		<Extension()>
		Public Function DataRow(dgv As DataGridView, rowIndex As Integer) As DataRow
			If rowIndex < 0 Then Return Nothing
			Dim itm = DirectCast(dgv.Rows(rowIndex).DataBoundItem, DataRowView)
			Return If(itm.Null, Nothing, itm.Row)
		End Function

		<Extension()>
		Public Function DataColumn(dgv As DataGridView, columnIndex As Integer) As DataColumn
			Dim tbl = DirectCast(dgv.DataSource, BindingSource).DataTable
			Dim propName = dgv.Columns(columnIndex).DataPropertyName
			If propName = "" Then Return Nothing
			Return tbl.Columns(propName)
		End Function

		<Extension()>
		Public Function Column(dgv As DataGridView, dataColumn As DataColumn) As DataGridViewColumn
			Return dgv.ColumnsX(dataColumn).First
		End Function
		<Extension()> _
      Public Function ColumnsX(dgv As DataGridView, dataColumn As DataColumn) As IEnumerable(Of DataGridViewColumn)
         Return dgv.Columns.Cast(Of DataGridViewColumn).Where(Function(col) col.DataPropertyName = dataColumn.ColumnName)
      End Function
#End Region 'DataGridView

#Region "TabControl"
      ''' <summary>
      ''' Dieser Schwachsinn im Form_Load ausgeführt behebt einen Bug beim DataBinding auf beim Start nicht sichtbaren Tabpages.
      ''' </summary>
      <Extension()> _
      Public Sub InitBindability(tc As TabControl)
         Dim parent = tc.FindForm
         parent.SuspendLayout()
         For Each tp As TabPage In tc.TabPages
            tp.SuspendLayout()
            For Each ctl In tp.Controls.Cast(Of Control).ToArray
               parent.Controls.Add(ctl)
               tp.Controls.Add(ctl)
            Next
            tp.ResumeLayout()
         Next
         parent.ResumeLayout()
      End Sub

      ''' <summary>hides Riders of TabControl</summary>
      <Extension()> _
      Public Sub HideRiders(tc As TabControl)
         tc.SuspendLayout()
         tc.SizeMode = TabSizeMode.Fixed
         tc.ItemSize = New Size(0, 1)
         tc.Appearance = TabAppearance.Buttons
         tc.ResumeLayout()
      End Sub
#End Region 'TabControl

		<Extension()>
		Public Function FindAncestor(Of T As Control)(ctl As Control) As T
			Do
				ctl = ctl.Parent
				FindAncestor = TryCast(ctl, T)
				If ctl Is Nothing OrElse FindAncestor IsNot Nothing Then Exit Do
			Loop
		End Function


	End Module

End Namespace
